#pragma rtGlobals=1		// Use modern global access method.

Function SetCalibrationPeriod(dataFolder, period, periodCenterTime, periodDuration, periodSolution)
	String dataFolder				// The data folder containing the globals to set.
	Variable period					// Number of period. Zero-based.
	Variable periodCenterTime		// Center time of the period in minutes.
	Variable periodDuration			// Duration of the end of this period in minutes.
	Variable periodSolution			// Solution in millimols. e.g., 0, 30, 100

	String saveDataFolder = GetDataFolder(1)
	SetDataFolder(dataFolder)

	NVAR gNumPeriods = gNumPeriods
	if (period >= gNumPeriods)					// Valid period?
		DoAlert 0, "SetCalibrationPeriod - period out of range"
	else
		WAVE periodInfo=periodInfo				// Get reference to periodInfo wave in the data folder.
		if (WaveExists(periodInfo))			// It should always exist.
			periodInfo[period][0] = periodCenterTime
			periodInfo[period][1] = periodDuration
			periodInfo[period][2] = periodSolution
		else
			DoAlert 0, "SetCalibrationPeriod - periodInfo wave does not exist"
		endif
	endif

	SetDataFolder(saveDataFolder)
End

Function CantFindPeriodInfoAlert(dataFolder)
	String dataFolder

	String message, leafName
	leafName = GetLeafName(dataFolder)
	sprintf message, "Can't find periodInfo wave for the %s run", leafName
	DoAlert 0, message
End

//	RemoveCenterTimeIndicator()
//	Removes the period center time indicator from the top graph.
Function RemoveCenterTimeIndicator()
	String graphName = WinName(0, 1)									// Name of the top graph.
	if (!IsRatioImgGraph(graphName))
		return -1
	endif
	Wave centerTimeIndicator = root:Packages:RatioImg:centerTimeIndicator	// Get a reference to the wave.
	if (!WaveExists(centerTimeIndicator))
		return -1
	endif
	CheckDisplayed centerTimeIndicator										// Is the wave on the graph?
	if (V_flag != 0)
		RemoveFromGraph centerTimeIndicator
	endif
End

//	MoveCenterTimeIndicator()
//	This works on the active graph. It moves cursor A to show the center time for the period
//	displayed in the Set Period Info control panel.
Function MoveCenterTimeIndicator()
	NVAR gPeriod = root:Packages:RatioImg:gPeriod
	
	String graphName = WinName(0, 1)						// Name of the top graph.
	if (!IsRatioImgGraph(graphName))
		return -1
	endif
	
	// Don't display the indicator if the Set Period Info panel is closed.
	DoWindow SetPeriodInfo										// Sets V_flag.
	if (V_flag == 0)
		return -1												// The control panel is not open.
	endif
	
	// Get the data folder for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(0)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		return -1
	endif
	
	// Find the ratio wave.
	Variable i = 0
	do
		Wave w = WaveRefIndexed("", i, 1)
		if (!WaveExists(w))									// No more waves.
			return -1
		endif
		if (IsRatioImgWaveOfType(w, "R"))
			break
		endif
		i += 1
	while(1)
	
	String traceName = WaveToTraceName("", w)
	if (strlen(traceName) == 0)
		return -1												// Should not happen.
	endif
	Wave periodInfo = $(dataFolder + "periodInfo")				// Get reference to the periodInfo wave.
	Variable centerTime = periodInfo[gPeriod][0]				// Get the center time for this period.
	GetAxis/Q left												// Sets local variables V_min and V_max.
	Wave indicator = root:Packages:RatioImg:centerTimeIndicator	// Get a reference to the wave.
	if (!WaveExists(indicator))
		return -1												// Should not happen.
	endif
	indicator = {V_min, V_max}
	SetScale x centerTime, 1.0001*centerTime, indicator
	CheckDisplayed/W=$graphName indicator	  				// Is the wave not yet on the graph?
	if (V_flag == 0)
		AppendToGraph/L=left indicator
		ModifyGraph rgb(centerTimeIndicator)=(39321,39321,39321)
		ModifyGraph live(centerTimeIndicator) = 1				// For less flashing when it is moved.
	endif
End

//	SetPeriodProc(ctrlName,popNum,popStr)
//	This is the action procedure for the Period control in the Set Period Info control panel.
//	It stores the newly selected period in the associated global variable and then
//	updates the values displayed in the other controls.
Function SetPeriodProc(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	
	NVAR gPeriod = root:Packages:RatioImg:gPeriod				// Create reference to global variable.

	// Make sure period is a valid value.
	gPeriod = round(gPeriod)									// Make sure it is an integer.
	if (gPeriod<0)
		gPeriod = 0
	endif
	if (gPeriod>9)												
		gPeriod = 9
	endif
	
	// These globals are set by the other controls in the control panel.
	NVAR gPeriodCenterTime = root:Packages:RatioImg:gPeriodCenterTime
	NVAR gPeriodDuration = root:Packages:RatioImg:gPeriodDuration
	NVAR gPeriodSolution = root:Packages:RatioImg:gPeriodSolution
	
	// Set the periodInfo wave for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		return -1
	endif
	
	String periodInfoStr = dataFolder + "periodInfo"				// Full path to this run's periodInfo wave.
	WAVE periodInfo = $periodInfoStr							// Get a reference to the wave.
	if (!WaveExists(periodInfo))
		CantFindPeriodInfoAlert(dataFolder)
		return -1
	endif
	
	// Set the global variables. This will cause the controls to update.
	gPeriodCenterTime = periodInfo[gPeriod][0]
	gPeriodDuration = periodInfo[gPeriod][1]
	gPeriodSolution = periodInfo[gPeriod][2]
	
	MoveCenterTimeIndicator()									// Display a visual indication of the center time.
End

//	StorePeriodInfoInDisplayedRun()
//	Use with the Set Period Info control panel.
//	Stores the values of the global variables for the center time, duration, and solution
//	in the periodInfo wave for the run displayed in the top graph.
Function StorePeriodInfoInDisplayedRun()
	// Get the period that we are to set.
	NVAR period = root:Packages:RatioImg:gPeriod
	
	// Get the new settings for the period.
	NVAR periodCenterTime = root:Packages:RatioImg:gPeriodCenterTime
	NVAR periodDuration = root:Packages:RatioImg:gPeriodDuration
	NVAR periodSolution = root:Packages:RatioImg:gPeriodSolution
	
	// Get the periodInfo wave for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		return -1
	endif
	
	String periodInfoStr = dataFolder + "periodInfo"				// Full path to this run's periodInfo wave.
	WAVE periodInfo = $periodInfoStr							// Get a reference to the wave.
	if (!WaveExists(periodInfo))
		CantFindPeriodInfoAlert(dataFolder)
		return -1
	endif
	
	SetCalibrationPeriod(dataFolder, period, periodCenterTime, periodDuration, periodSolution)
End

//	SetPeriodInfoProc(ctrlName)
//	This is the action procedure for the SetVariable controls in the Set Period Info control panel.
//	It stores the center, duration and solution values in the periodInfo wave for the
//	currently graphed run.
Function SetPeriodInfoProc(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	
	StorePeriodInfoInDisplayedRun()
	if (CmpStr(ctrlName, "PeriodCenter") == 0)			// User changed the center time?
		MoveCenterTimeIndicator()							// Display a visual indication of the center time.
	endif
End

Function SetFromCursorButtonProc(ctrlName) : ButtonControl
	String ctrlName
		
	// Get the period that we are to set.
	NVAR period = root:Packages:RatioImg:gPeriod
	
	// Get a reference to the center time global variable.
	NVAR periodCenterTime = root:Packages:RatioImg:gPeriodCenterTime
	
	// Get the x coordinate of cursor A (the round one).
	if (strlen(CsrWave(A)) == 0)
		DoAlert 0, "Put cursor A (the round one) on the graph before clicking Set From Cursor"
		return -1
	endif
	Variable cursorTime = xcsr(A)
	
	// Set the global variable to the value.
	periodCenterTime = cursorTime
	
	// Store the new value in the periodInfo wave.
	StorePeriodInfoInDisplayedRun()
	MoveCenterTimeIndicator()							// Display a visual indication of the center time.
End

//	UpdateSetPeriodInfoPanel()
//	This is called when a RatioImg graph is activated. If the Set Period Info control panel
//	is displayed, this routine updates the displayed values to reflect the newly-activated graph.
//	If it is not displayed, we still set the global variables so that the panel will be correct
//	if the user opens it later.
Function UpdateSetPeriodInfoPanel()
	// Create references to the global variables tied to the controls in the panel.
	NVAR period = root:Packages:RatioImg:gPeriod
	NVAR periodCenterTime = root:Packages:RatioImg:gPeriodCenterTime
	NVAR periodDuration = root:Packages:RatioImg:gPeriodDuration
	NVAR periodSolution = root:Packages:RatioImg:gPeriodSolution
	
	// Get the periodInfo wave for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		return -1
	endif
	
	String periodInfoStr = dataFolder + "periodInfo"				// Full path to this run's periodInfo wave.
	WAVE periodInfo = $periodInfoStr							// Get a reference to the wave.
	if (!WaveExists(periodInfo))
		return -1
	endif
	
	// Set the global variables.
	periodCenterTime = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	periodSolution = periodInfo[period][2]
End

//	SetDefaultPeriodInfoButtonProc(ctrlName)
//	This is the action procedure for the Set As Defaults [Set Calibration Period panel] button.
//	It sets the default period info for new runs to be the same as the period info for
//	the currently displayed run.
Function SetDefaultPeriodInfoButtonProc(ctrlName) : ButtonControl
	String ctrlName
	
	// Get the periodInfo wave for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		return -1
	endif
	
	String periodInfoStr = dataFolder + "periodInfo"				// Full path to this run's periodInfo wave.
	WAVE periodInfo = $periodInfoStr							// Get a reference to the wave.
	if (!WaveExists(periodInfo))
		CantFindPeriodInfoAlert(dataFolder)
		return -1
	endif
	
	Duplicate/O periodInfo, root:Packages:RatioImg:periodInfo
	NVAR numPeriods = root:Packages:RatioImg:gNumPeriods
	numPeriods = DimSize(periodInfo, 0)					// Number of periods = number of rows in periodInfo wave.
End

//	SetPeriodInfoHook(infoStr)
//	This routine is the hook function for the Set Period Info control panel. It is called when certain events
//	(e.g., activate, deactivate, kill) occur.
Function SetPeriodInfoHook(infoStr)
	String infoStr
	
	Variable statusCode = 0									// 0 if nothing done, else 1
	
	String eventKey = StringByKey("EVENT", infoStr)		// Determine which event happened.
	
	if (CmpStr(eventKey, "activate") == 0)
		// If the Set Period Info control panel is displayed, we need to update what it is displaying.
		MoveCenterTimeIndicator()							// Display a visual indication of the center time.
	endif
	
	if (CmpStr(eventKey, "deactivate") == 0)
		RemoveCenterTimeIndicator()						// Remove the period indicator line from the active graph.
	endif
	
	if (CmpStr(eventKey, "kill") == 0)
		RemoveCenterTimeIndicator()						// Remove the period indicator line from the active graph.
	endif

	return statusCode
End

Function CorrectRatiosButtonProc(ctrlName) : ButtonControl
	String ctrlName
	// Get the periodInfo wave for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		DoAlert 0, "Can't find a RatioImg graph"
		return -1
	endif
	CorrectRatioImgRatios(dataFolder)
End

Function CalculateFFoProc(ctrlName) : ButtonControl
	String ctrlName
	// Get the periodInfo wave for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		DoAlert 0, "Can't find a RatioImg graph"
		return -1
	endif
	CorrectFFos(dataFolder)
End

Function CalculateDeltaFFProc(ctrlName) : ButtonControl
	String ctrlName
	// Get the periodInfo wave for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		DoAlert 0, "Can't find a RatioImg graph"
		return -1
	endif
	CorrectDeltaFFs(dataFolder)
End


//	SetPeriodInfo()
//	Creates the SetPeriodInfo control panel.
Window SetPeriodInfo() : Panel
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=1 /W=(140,531,435,628) as "Set Calibration Periods"
	SetDrawLayer UserBack
	SetDrawEnv linefgc= (56797,56797,56797),fillfgc= (57906,61602,58830)
	DrawRect -2,-2,317,133
	SetDrawEnv linefgc= (65535,65535,65535),fillfgc= (57906,61602,58830)
	DrawRect 9,2,161,90
	DrawRect 163,120,163,115
	SetDrawEnv fillfgc= (57906,61602,58830)
	SetDrawEnv save
	SetVariable CurrentPeriod,pos={50,8},size={104,17},proc=SetPeriodProc,title=" Period:"
	SetVariable CurrentPeriod,help={"Choose the calibration period to which the other settings apply."}
	SetVariable CurrentPeriod,limits={0,10,1},value= root:Packages:RatioImg:gPeriod
	SetVariable PeriodCenter,pos={14,48},size={140,17},proc=SetPeriodInfoProc,title=" Center Time:"
	SetVariable PeriodCenter,help={"Enter the center of the calibration period in minutes."}
	SetVariable PeriodCenter,limits={0,200,0.166667},value= root:Packages:RatioImg:gPeriodCenterTime
	SetVariable PeriodDuration,pos={37,68},size={117,17},proc=SetPeriodInfoProc,title=" Duration:"
	SetVariable PeriodDuration,help={"Enter the duration of the calibration period in minutes."}
	SetVariable PeriodDuration,limits={0,200,0.166667},value= root:Packages:RatioImg:gPeriodDuration
	SetVariable PeriodSolution,pos={40,28},size={114,17},proc=SetPeriodInfoProc,title=" Solution:"
	SetVariable PeriodSolution,help={"Enter the solution concentration of the calibration period in millimoles."}
	SetVariable PeriodSolution,limits={0,150,5},value= root:Packages:RatioImg:gPeriodSolution
	Button SetFromCursor,pos={168,8},size={108,17},proc=SetFromCursorButtonProc,title="Set From Cursor"
	Button SetFromCursor,help={"Sets the center time for the selected calibration period from cursor A (the round cursor)."}
	Button SetDefaultPeriodInfo,pos={168,28},size={108,17},proc=SetDefaultPeriodInfoButtonProc,title="Set As Defaults"
	Button SetDefaultPeriodInfo,help={"Sets the default calibration period information for new runs to be the same as that for the currently displayed run."}
	Button CalibrateGraph,pos={168,48},size={108,17},proc=CorrectRatiosButtonProc,title="Correct Ratios"
	Button CalibrateGraph,help={"Corrects the ratios for waves in the active graph using the current panel settings."}
	Button CalibrateGraph_1,pos={168,68},size={52,17},proc=CalculateFFoProc,title="F/Fo"
	Button CalibrateGraph_1,help={"Calculates and plots fluorescence values relative to baseline for waves in the active graph using the current panel settings for Period 0."}
	Button CalibrateGraph_2,pos={224,68},size={52,17},proc=CalculateDeltaFFProc,title="dF/F"
	Button CalibrateGraph_2,help={"Calculates and plots fluorescence values relative to baseline - 1 for waves in the active graph using the current panel settings for Period 0."}
	SetWindow kwTopWin,hook=SetPeriodInfoHook
EndMacro

//	DisplaySetPeriodInfoPanel()
//	Brings the Set Period Info control panel to the front after creating it if necessary.
Function DisplaySetPeriodInfoPanel()
	DoWindow/F SetPeriodInfo
	if (V_flag == 0)
		Execute "SetPeriodInfo()"		// We need to use Execute because you can't directly call a macro from a function.
	endif
End
