#pragma rtGlobals=1		// Use modern global access method.

// This file contains routines to load data from RatioImg files, to create the ratio waves, and to correct them.
// The default values for gStartLine and the routines for handling "Event" markers are specific for
// Georgia Instruments data acquisition software.

//	LoadRatioImgFile()
//	Loads data from a RatioImg file creating new waves in a new data folder in the current data folder.
//	The waves are named as follows:
//		bkgA:		The wavelength A background (from ROI 1)
//		bkgB:		The wavelength B background (from ROI 1)
//		A2:			Wavelength A data from ROI 2.
//		B2:			Wavelength B data from ROI 2.
//		A3, B3, A4, B4, etc. All waves from A2/B2 onward may have the A or B background values subtracted, 
//		depending on the specifications in the Set RatioImg Settings dialog (see below).
//	
//	This routine also creates a global variable in the data folder named gNumDataPairs,
//	which contains the number of A/B pairs loaded.
//
//	The file name is assumed to have the form "981209.csv". The new data folder will
//	have the name F981209.
//
//	This uses three globals, which can be set through the Set RatioImg Settings dialog, to control its behavior.
//	gStartLine is the line number (counting from 0) of the first line that contains data to be loaded. 
//	This would normally be 2. but since users normally count from 1, not 0, this is set to 3. 
//       Not the perfect solution since it drops the first data point. 
//
//	If gMakeTable is non-zero, a table of the data is created. This will usually be
//	of interest only during procedure development. If gMakeTable is zero,
//	the data is loaded into memory but not yet displayed. This would be the normal
//	mode of operation.
//
//	If gDoBkgSub is non-zero, background subtraction will be done for each A/B
//	pair. If it is zero, no background subtraction will be done. Normally, background subtraction
//	should be done but some data files don't have valid backgrounds. The first pair of waves in the data
//	set are assumed to be from an ROI placed on a background  (i.e., non-cell) area in the image field.
//	If this is not the case you may either edit the data with a spreadsheet program accordingly, or you may leave
//	background subtraction disabled (which is currently the default setting).
//
//	After loading the data, LoadRatioImgFile scans through the first column looking for blanks.
//	A blank will occur if the line contains "Event" instead of valid numbers. This indicates
//	that data acquisition was stopped and restarted; i.e., a false-start occurred. After finding
//	the last blank in the first wave, LoadRatioImgFile deletes this blank and all preceding points
//	in all of the waves. The "Event" at the end of the file is not loaded as a blank because of the 
//	use of the /V in the LoadWave operation. The last "1" in this flag tells LoadWave to ignore 
//	trailing blanks.

Function LoadRatioImgFile()
	Variable firstDataLine = NumVarOrDefault("root:Packages:RatioImg:gFirstDataLine", 3)
	Variable makeTable = NumVarOrDefault("root:Packages:RatioImg:gMakeTable", 0)
	Variable doBkgSub = NumVarOrDefault("root:Packages:RatioImg:gDoBkgSub", 0)

	String originalCurrentDataFolder= GetDataFolder(1)	// Includes trailing colon.
	
	String dataFolderName = UniqueName("F", 11, 0)		// This is a temporary data folder name.
	CreateNewRatioImgDataFolder(dataFolderName)
	
	LoadWave/J/Q/N=tempRatioImg/E=(makeTable)/K=1/V={"\t,"," $",0,1}/L={0,firstDataLine,0,0,0} ""
	
	Variable numWaves = V_flag							// V_flag is set by LoadWave.
	if (numWaves == 0)									// No waves loaded? User cancelled?
		KillDataFolder :										// Kill the current data folder, which is the new data folder.
		SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
		return -1
	endif
	
	// Find any blanks in the first wave loaded and use this to strip out false-start data from all of the waves.
	Wave w = tempRatioImg0								// Get a reference to the first loaded wave.
	Variable lastBlankPoint = -1							// location of the last blank point in the wave.
	Variable numPoints = numpnts(w)						// Number of points in the wave.
	Variable pn = 0											// pn == point number
	do
		if (numtype(w[pn]) == 2)							// This point contains a NaN?
			lastBlankPoint = pn
		endif
		pn += 1
	while(pn < numPoints)
	if (lastBlankPoint >= 0)								// Found blanks?
		Variable wn = 0										// wn == wave number
		do
			Wave w = $("tempRatioImg" + num2istr(wn))		// tempRatioImg0, tempRatioImg1, . . .
			DeletePoints 0, lastBlankPoint+1, w
			wn += 1
		while(wn < numWaves)
	endif
	
	// This stores a global variable in the current data folder that will tell us how many A/B pairs we have.
	
	Variable/G gNumDataPairs = numWaves / 2
	
	// Rename the data folder using the file name.
	// NOTE: This assumes that the file name is of the form "981209.csv".
	Variable pos
	pos = strsearch(UpperStr(S_fileName), ".CSV", 0)
	if (pos >= 1)															// We found the ".csv" ?
		String newDataFolderName = "R" + S_fileName[0, pos-1]
		if (DataFolderExists("::" + newDataFolderName))					// Already exists?
			newDataFolderName = UniqueName(newDataFolderName, 6, 1)	// Find a name that does not conflict.
		endif
		RenameDataFolder :, $newDataFolderName
		dataFolderName = newDataFolderName
	endif
	
	String dataFolder = originalCurrentDataFolder + dataFolderName			// Full path to the new data folder without trailing colon.
	
	// Rename the background waves.
	Rename tempRatioImg0, bkgA
	Rename tempRatioImg1, bkgB
	
	// Rename the data waves as Ai or Bi and do the background subtraction and set the wave's X scaling.
	String oldName, newName
	Variable i = 2
	do
		oldName = StringFromList(i, S_waveNames, ";")
		if (strlen(oldName) == 0)											// No more waves
			break
		endif
		Wave w = $oldName
		if (i %& 1)
			Wave bkg = bkgB
			newName = "B"
		else
			Wave bkg = bkgA
			newName = "A"
		endif
		if (doBkgSub)
			w -= bkg														// Background subtraction
		endif
		newName += num2istr(trunc(i/2)+1) 								
		Rename $oldName, $newName
		NVAR samplesPerMinute = gSamplesPerMinute						// Create reference to global variable in the current data folder.
		SetScale/P x 0, 1/samplesPerMinute, "Minutes", $newName			// Set wave's X scaling.		
		i += 1
	while(1)
	
	// Create the ratio waves, Ri=Ai/Bi.
	CreateRatioImgRatios(dataFolder)
	
	SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
End

Function LoadSingleROIDataFile()
	Variable firstDataLine = NumVarOrDefault("root:Packages:RatioImg:gFirstDataLine", 0)
	//makeTable is disabled in this function because it conflicts with the use of KillWaves 
	Variable doBkgSub = NumVarOrDefault("root:Packages:RatioImg:gDoBkgSub", 0)

	String originalCurrentDataFolder= GetDataFolder(1)	// Includes trailing colon.
	
	String dataFolderName = UniqueName("F", 11, 0)		// This is a temporary data folder name.
	CreateNewRatioImgDataFolder(dataFolderName)
	
	LoadWave/J/Q/N=A/K=1/V={"\t,"," $",0,1}/L={0,firstDataLine,0,0,0} ""
	Variable loadNumpts = numpnts (A0)					// Assumes that all waves are the same length as the first one in the data file.
		
	Variable numWaves = V_flag							// V_flag is set by LoadWave.
		
	if (numWaves == 0)									// No waves loaded? User cancelled?
		KillDataFolder :										// Kill the current data folder, which is the new data folder.
		SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
		return -1
	endif
	
	Variable i = 1
	do
		String BName = "B" + num2str(i)
		String oldAName = "A" + num2str(numwaves - (i))
		String newAName = "A" + num2str(numWaves - (i-1))
		Duplicate/O $oldAName, $newAName; KillWaves $oldAName
		if (i == 1)
			Make/O/N = (loadNumpts) $("B" + num2str(i))
		else
			Make/O/N = (loadNumpts) $("B" + num2str(i))
			Wave w = $BName
			w = 1
		endif
		i += 1
	while (i <= numWaves)
	
	// This stores a global variable in the current data folder that will tell us how many A/B pairs we have.
	
	Variable/G gNumDataPairs = numWaves	
	
	// Rename the data folder using the file name.
	// NOTE: This assumes that the file name is of the form "981209.csv".
	Variable pos
	pos = strsearch(UpperStr(S_fileName), ".CSV", 0)
	if (pos >= 1)															// We found the ".csv" ?
		String newDataFolderName = "R" + S_fileName[0, pos-1]
		if (DataFolderExists("::" + newDataFolderName))					// Already exists?
			newDataFolderName = UniqueName(newDataFolderName, 6, 1)	// Find a name that does not conflict.
		endif
		RenameDataFolder :, $newDataFolderName
		dataFolderName = newDataFolderName
	endif
	
	String dataFolder = originalCurrentDataFolder + dataFolderName			// Full path to the new data folder without trailing colon.
	
	// Rename the background waves.
	Rename A1, bkgA
	Rename B1, bkgB
	Wave bkg = bkgA
	
	// Do the background subtraction.
	String workName
	i = 2				
	do
		workName = "A" + num2str(i)
		Wave w = $workName
		if (doBkgSub)
			w -= bkg														// Background subtraction
		endif
		i += 1
	while(i <= numWaves)
	
	// Create the ratio waves, Ri=Ai/Bi.
	CreateRatioImgRatios(dataFolder)
	
	// Kill the A and B waves and set the R wave scaling
	NVAR samplesPerMinute = gSamplesPerMinute							// Create reference to global variable in the current data folder.
	i = 2
	do
		String killAName = "A" + num2str(i)
		String killBName = "B" + num2str(i)
		String scaleRName = "R" + num2str(i)
		KillWaves $killAName
		KillWaves $killBName
		SetScale/P x 0, 1/samplesPerMinute, "Minutes", $scaleRName
		i += 1
	while (i <= numWaves)
	
	SetDataFolder(originalCurrentDataFolder)								// Restore the current data folder.
	
End

Function CorrectFFo(cell)
	Variable cell										
	
	WAVE R = $("R"+num2istr(cell))					// Get a reference to the ratio wave (R2, R3 . . .). 
	WAVE periodInfo = periodInfo						// Get a reference to the run global period information.
	
	// Find average value for period 0.
	
	Variable period, periodCenter, periodDuration, periodStart, periodEnd
	Variable avg0
	
	period = 0
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg0 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg0
	
Variable n = numpnts(R)
	Variable i = 0
	Variable val
	do
		val = R[i]
		val = val / avg0
		R[i] = val
		i += 1
	while(i < n)
	
	return 0
End

Function CorrectDeltaFF(cell)
	Variable cell										
	
	WAVE R = $("R"+num2istr(cell))					// Get a reference to the ratio wave (R2, R3 . . .). 
	WAVE periodInfo = periodInfo						// Get a reference to the run global period information.
	
	// Find average value for period 0.
	
	Variable period, periodCenter, periodDuration, periodStart, periodEnd
	Variable avg0
	
	period = 0
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg0 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg0
	
Variable n = numpnts(R)
	Variable i = 0
	Variable val
	do
		val = R[i]
		val = (val / avg0) -1
		R[i] = val
		i += 1
	while(i < n)
	
	return 0
End

//	CreateRatioImgRatios(dataFolder)
//	The specified data folder is assumed to contain the Ai/Bi waves loaded by the LoadRatioImgFile procedure.
//	This routine creates the corresponding Ri wave and sets it to Ai/Bi.
Function CreateRatioImgRatios(dataFolder)
	String dataFolder									// Data folder containing RatioImg data for which ratios are to be created.
	
	String originalCurrentDataFolder=GetDataFolder(1)	// Includes trailing colon.
	SetDataFolder(dataFolder)							// Make this the current data folder.

	NVAR numDataPairs=gNumDataPairs				// Access the global variable named gNumDataPairs in the current data folder.
	String aName, bName, rName
	Variable i = 2										
	do
		aName = "A" + num2istr(i)						// e.g., "A2"
		Wave A = $aName								// Get a reference to the A wave
		bName = "B" + num2istr(i)						// e.g., "B2"
		Wave B = $bName								// Get a reference to the B wave
		rName = "R" + num2istr(i)						// Ratio wave will be named R2
		Duplicate/O A, $rName
		Wave R = $rName								// Get a reference to the R wave
		R = A / B										// Do the ratio
		i += 1
	while(i <= numDataPairs)
	
	SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
End

//	CorrectRatioImgRatio(cell)
//	Corrects the ratio wave Ri where i=cell.
//	This function works on the current data folder.
//	After doing the correction, this procedure searches all RatioImg graphs looking to see
//	if the correct wave is displayed. If it is displayed, it changes the left axis range.
//	This is necessary because the axis range for the raw ratio is not appropriate for the
//	corrected ratio.
Function CorrectRatioImgRatio(cell)
	Variable cell										// 1-based cell to create calibrated curve for.
	
	WAVE R = $("R"+num2istr(cell))					// Get a reference to the ratio wave (R2, R3 . . .). 
	WAVE periodInfo = periodInfo						// Get a reference to the run global period information.
	
	// Find average value for each period.
	
	Variable period, periodCenter, periodDuration, periodStart, periodEnd, solution
	Variable avg0, avg1, avg2, avg3, avg4, avg5, avg6, avg7, avg8, avg9
	Variable solution0, solution1, solution2, solution3, solution4, solution5, solution6, solution7, solution8, solution9
	
	period = 0
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution0 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg0 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg0
	
	period = 1
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution1 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg1 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg1
	
	period = 2
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution2 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg2 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg2
	
	period = 3
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution3= periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg3 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg3
	
	period = 4
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution4 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg4 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg4
	
	period = 5
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution5 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg5 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg5
	
	period = 6
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution6 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg6 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg6
	
	period = 7
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution7 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg7 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg7
	
	period = 8
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution8 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg8 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg8
	
	period = 9
	periodCenter = periodInfo[period][0]
	periodDuration = periodInfo[period][1]
	solution9 = periodInfo[period][2]
	periodStart = periodCenter - periodDuration/2
	periodEnd = periodCenter + periodDuration/2
	avg9 = mean(R, periodStart, periodEnd)
	periodInfo[period][3] = avg9
	
	if (RatioImgDebug())
		Printf "Cell %d period averages = %g, %g, %g\r", cell, avg0, avg1, avg2, avg3, avg4, avg5, avg6, avg7, avg8, avg9
	endif
	
	// Find the slopes and intercepts.
	Variable slope0_1, intercept0_1, slope1_2, intercept1_2, slope2_3, intercept2_3, slope3_4, intercept3_4, slope4_5, intercept4_5, slope5_6, intercept5_6, slope6_7, intercept6_7, slope7_8, intercept7_8, slope8_9, intercept8_9

	slope0_1 = (solution1 - solution0) / (avg1 - avg0)
	intercept0_1 = solution1 - slope0_1*avg1
	periodInfo[0][4] = slope0_1
	periodInfo[0][5] = intercept0_1
	
	slope1_2 = (solution2 - solution1) / (avg2 - avg1)
	intercept1_2 = solution2 - slope1_2*avg2
	periodInfo[1][4] = slope1_2
	periodInfo[1][5] = intercept1_2
	
	slope2_3 = (solution3 - solution2) / (avg3 - avg2)
	intercept2_3 = solution3 - slope2_3*avg3
	periodInfo[2][4] = slope2_3
	periodInfo[2][5] = intercept2_3
	
	slope3_4 = (solution4 - solution3) / (avg4 - avg3)
	intercept3_4 = solution4 - slope3_4*avg4
	periodInfo[3][4] = slope3_4
	periodInfo[3][5] = intercept3_4
	
	slope4_5 = (solution5 - solution4) / (avg5- avg4)
	intercept4_5 = solution5 - slope4_5*avg5
	periodInfo[4][4] = slope4_5
	periodInfo[4][5] = intercept4_5
	
	slope5_6 = (solution6 - solution5) / (avg6 - avg5)
	intercept5_6 = solution6 - slope5_6*avg6
	periodInfo[5][4] = slope5_6
	periodInfo[5][5] = intercept5_6
	
	slope6_7 = (solution7 - solution6) / (avg7 - avg6)
	intercept6_7 = solution7 - slope6_7*avg7
	periodInfo[6][4] = slope6_7
	periodInfo[6][5] = intercept6_7
	
	slope7_8 = (solution8 - solution7) / (avg8 - avg7)
	intercept7_8 = solution8 - slope7_8*avg8
	periodInfo[7][4] = slope7_8
	periodInfo[7][5] = intercept7_8
	
	slope8_9 = (solution9 - solution8) / (avg9 - avg8)
	intercept8_9 = solution9 - slope8_9*avg9
	periodInfo[8][4] = slope8_9
	periodInfo[8][5] = intercept8_9
	
	
	if (RatioImgDebug())
		Printf "Cell %d slopes/intercepts = %g/%g, %g/%g\r", cell, slope0_1, intercept0_1, slope1_2, intercept1_2, slope2_3, intercept2_3, slope3_4, intercept3_4, slope4_5, intercept4_5, slope5_6, intercept5_6, slope6_7, intercept6_7, slope7_8, intercept7_8, slope8_9, intercept8_9
	endif
	
	Variable n = numpnts(R)
	Variable i = 0
	Variable val
	do
		val = R[i]
		if (val < avg1)									// e.g., < average for 10 millimolar solution
			val = val * slope0_1 + intercept0_1
		else
			if (val < avg2)
				val = val * slope1_2 + intercept1_2
			else
				if (val < avg3)
					val = val * slope2_3 + intercept2_3
				else 
					if (val < avg4)
						val = val * slope3_4 + intercept3_4
					else 
						if (val < avg5)
							val = val * slope4_5 + intercept4_5
						else
							if (val < avg6)
								val = val * slope5_6 + intercept5_6
							else
								if (val < avg7)
									val = val * slope6_7 + intercept6_7
								else
									if (val < avg8)
										val = val * slope7_8 + intercept7_8
									else
										if (val < avg9)
											val = val * slope8_9 + intercept8_9
										else
											val = val * slope8_9 + intercept8_9	// This uses the same correction factors as those of the previous interval in order to handle 
										endif										// values above the average of the highest calibration solution.
									endif
								endif
							endif
						endif
					endif
				endif
			endif
		endif
		R[i] = val
		i += 1
	while(i < n)
	
	return 0
End

//	CorrectRatioImgRatios(dataFolder)
//	Runs the ratio correction procedure for all Ai/Bi/Ri data sets in the specified data folder.
Function CorrectRatioImgRatios(dataFolder)
	String dataFolder
	String originalCurrentDataFolder=GetDataFolder(1)	// Includes trailing colon.
	SetDataFolder(dataFolder)							// Make this the current data folder.
	NVAR ratiosCorrected=gRatiosCorrected				// Access the global variable named gRatiosCorrected.
	if (ratiosCorrected != 0)
		String temp
		String leafName = GetLeafName(dataFolder)
		sprintf temp, "The values for %s have already been corrected.", leafName
		DoAlert 0, temp
		return -1
	endif
		
	DoAlert 1, "This will overwrite the ratio waves with the corrected value waves.  Please check your calibration period settings before proceeding.  Are you sure you want to do this?"
	if (V_flag == 1)
		NVAR numDataPairs=gNumDataPairs				// Access the global variable named gNumDataPairs.
	
		Variable i = 2										
		do
			CorrectRatioImgRatio(i)
			i += 1
		while(i <= numDataPairs)
		
		ratiosCorrected = 1									// Note that we have corrected the ratios.
		
		WavesAverage("R","R_Average")					// Make an averaged wave from all the corrected waves (see below)
		
		SetAxis/Z left -5,105								// Set axis for calibrations of 0 to 100 millimolar. NOTE: You will probably want to change these values based on the scale of ion concentrations used in your experiment.

		SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
	else
		print "Ratio correction cancelled."
		SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
	endif
End

// WavesAverage(baseName, destName)
// Produces a new wave, each point of which contains the average of the corresponding points of a number of source waves.
// All waves whose name starts with the specified base name are source waves.
// This function assumes that all waves that start with the base name have the same number of points and that there is at least one such wave.
Function WavesAverage(baseName, destName)
	String baseName // name for source wave
	String destName // name for destination wave
	
	String wn // contains the name of a particular wave
	String wl // contains a list of wave names
	Variable index=0
	
	// get list of waves whose names start with baseName
	wl = WaveList(baseName+"*", ";", "")
	
	// Make destination wave based on the first source wave
	wn = StringFromList(0, wl)
	
	Duplicate/O $wn, $destName
	Wave/D dest = $destName //create wave reference for destination
	dest = 0
	do
		wn = StringFromList(index, wl) // get next wave
		if (strlen(wn) == 0) // no more names in list?
			break // break out of loop
		endif
		Wave/D source = $wn // create wave reference for source
		dest += source // add source to dest
		index += 1
	while (1) // do unconditional loop
	
	dest /= index // divide by number of waves
End

//	CorrectFFos(dataFolder)
//	Performs the F/Fo calculation procedure for the R waves in the currently-displayed run.
Function CorrectFFos(dataFolder)
	String dataFolder
	String originalCurrentDataFolder=GetDataFolder(1)	// Includes trailing colon.
	SetDataFolder(dataFolder)							// Make this the current data folder.
	NVAR ratiosCorrected=gRatiosCorrected				// Access the global variable named gRatiosCorrected.
	if (ratiosCorrected != 0)
		String temp
		String leafName = GetLeafName(dataFolder)
		sprintf temp, "The values for %s have already been corrected.", leafName
		DoAlert 0, temp
		SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
		return -1
	endif
	
	NVAR doBkgSub=gDoBkgSub							// Access the global variable named gDoBkgSub.
	NVAR numDataPairs=gNumDataPairs				// Access the global variable named gNumDataPairs.
	
	DoAlert 1, "Please check your settings. Period 0 should be set to the Center Time and Duration for the fluorescence (F) baseline.  The data waves will be overwritten with F/Fo values.  Are you sure you want to do this?"
	if (V_flag == 1)
		if (doBkgSub == 1) 
			Variable i = 2										// Do the F/Fo calculation.
			do
				CorrectFFo(i)
				i += 1
			while(i <= numDataPairs)
		
			ratiosCorrected = 1									// Note that we have calculated the F/Fo ratios.
		
			SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
			SetAxis/Z left 0.2,2									// Set axis for F/Fo. NOTE: You will probably want to set these values to bracket the typical ion changes that occur in your experiments.
			Print "F/Fo calculation complete (background subtraction performed during data loading)."
			return -1
		endif
		if (doBkgSub == 0)
			DoAlert 1, "This datafile was loaded without background subtraction enabled. Proceed to calculate F/Fo without background subtraction?"
			if (V_flag == 1)
				i = 2												// Do the F/Fo calculation.
				do
					CorrectFFo(i)
					i += 1
				while(i <= numDataPairs)
		
				ratiosCorrected = 1									// Note that we have calculated the F/Fo ratios.
		
				SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
				SetAxis/Z left 0.2,2									// Set axis for F/Fo. NOTE: You will probably want to set these values to bracket the typical proportional ion changes that occur in your experiments.
				Print "F/Fo calculation performed without background subtraction."
			endif
			if (V_flag == 3)
				print "F/Fo calculation cancelled."
				SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
				return -1
			endif
		endif
		WavesAverage("R","R_Average")					// Make an averaged wave from all the corrected waves (see below)
	endif
	print  "F/Fo calculation cancelled."
	SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
End

//	CorrectDeltaFFs(dataFolder)
//	Performs the Delta F/F calculation procedure for the R waves in the currently-displayed run.
Function CorrectDeltaFFs(dataFolder)
	String dataFolder
	String originalCurrentDataFolder=GetDataFolder(1)	// Includes trailing colon.
	SetDataFolder(dataFolder)							// Make this the current data folder.
	NVAR ratiosCorrected=gRatiosCorrected				// Access the global variable named gRatiosCorrected.
	if (ratiosCorrected != 0)
		String temp
		String leafName = GetLeafName(dataFolder)
		sprintf temp, "The values for %s have already been corrected.", leafName
		DoAlert 0, temp
		SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
		return -1
	endif
	
	NVAR doBkgSub=gDoBkgSub							// Access the global variable named gDoBkgSub.
	NVAR numDataPairs=gNumDataPairs				// Access the global variable named gNumDataPairs.
	
	DoAlert 1, "Please check your settings. Period 0 should be set to the Center Time and Duration for the fluorescence (F) baseline.  The data waves will be overwritten with dF/F values.  Are you sure you want to do this?"
	if (V_flag == 1)
		if (doBkgSub == 1) 
			Variable i = 2										// Do the dF/F calculation.
			do
				CorrectDeltaFF(i)
				i += 1
			while(i <= numDataPairs)
		
			ratiosCorrected = 1									// Note that we have calculated the dF/F ratios.
		
			SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
			SetAxis/Z left 0.2,2									// Set axis for dF/F. NOTE: You will probably want to set these values to bracket the typical ion changes that occur in your experiments.
			Print "dF/F calculation complete (background subtraction performed during data loading)."
			return -1
		endif
		if (doBkgSub == 0)
			DoAlert 1, "This datafile was loaded without background subtraction enabled. Proceed to calculate dF/F without background subtraction?"
			if (V_flag == 1)
				i = 2												// Do the dF/F calculation.
				do
					CorrectDeltaFF(i)
					i += 1
				while(i <= numDataPairs)
		
				ratiosCorrected = 1									// Note that we have calculated the dF/F ratios.
		
				SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
				SetAxis/Z left -0.2,2								// Set axis for dF/F. NOTE: You will probably want to set these values to bracket the typical proportional ion changes that occur in your experiments.
				Print "dF/F calculation performed without background subtraction."
			endif
			if (V_flag == 3)
				print "dF/F calculation cancelled."
				SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
				return -1
			endif
		endif
		WavesAverage("R","R_Average")					// Make an averaged wave from all the corrected waves (see below)
	endif
	print  "dF/F calculation cancelled."
	SetDataFolder(originalCurrentDataFolder)			// Restore the current data folder.
End

//	CorrectRatioImgRatiosFromList(dataFolderList)
//	Runs the ratio correction procedure for all Ai/Bi/Ri data sets in the listed data folders.
//	If dataFolderList is "", it works on the run data folders selected in the Data Browser.
Function CorrectRatioImgRatiosFromList(dataFolderList) : ButtonControl
	String dataFolderList
	
	if (strlen(dataFolderList) == 0)
		dataFolderList = GetSelectedRunDataFolders(0)
	endif
	
	if (strlen(dataFolderList) == 0)
		DoAlert 0, "Select one or more run data folders in the Data Browser before invoking this procedure"
		return -1
	endif
	
	String dataFolder
	Variable i = 0
	do
		dataFolder = StringFromList(i, dataFolderList, ";")
		if (strlen(dataFolder) == 0)
			break												// No more run data folders.
		endif
		CorrectRatioImgRatios(dataFolder)
		i += 1
	while(1)
	
	return 0	
End
