#pragma rtGlobals=1		// Use modern global access method.

#include "RatioImg Utilities"
#include "RatioImg Data Loading"
#include "RatioImg Calibration"
#include "RatioImg Graphs"
#include "RatioImg Measurements"

Menu "RatioImg"
	help = {"RatioImg menu.\r\rUse this to load, display, and manipulate RatioImg data."}

	"Initialize/0", InitRatioImgGlobals()
	help = {"Creates global variables used by RatioImg procedures.\r\rThis needs to be run once when RatioImg procedures are used in a new Igor experiment."}

	"Data Loading...", MSetRatioImgDataLoadingSettings()
	help = {"Displays a dialog for entering display-related settings used in making RatioImg graphs."}
		
	"Axis Ranges...", MAutoSetRatioImgAxisRanges()
	help = {"Displays a dialog for setting axis ranges in RatioImg graphs."}
	
	"Load Data.../1", LoadRatioImgFile()
	help = {"Loads data from a RatioImg file, creating a new run data folder.\r\rCreates ratio waves and corrects them."}

	"Create Graphs/2", CreateRatioImgGraphsFromList("")	
	help = {"Creates a graph for each run data folder selected in the Data Browser."}

	"Correct Ratios", CorrectRatioImgRatiosFromList("")
	help = {"Corrects the ratio waves for each run data folder selected in the Data Browser."}
	
	"-"

	"View Measurements/3", ViewMeasurements()
	help = {"Displays measurements in a table."}

	"Add a Measurement/4", AddMeasurementFromCursors("")
	help = {"Adds measurement based on cursor A and cursor B on the top graph."}

	"Sort Measurements/5", SortMeasurements()
	help = {"Sorts all measurements based on the name of the measurement and the start time."}
		
	"-"
	
	"Graph Avg/6",MakeAvgWaveGraph("")
	help={"Makes a new graph showing the average of all the R traces in the currently-selected corrected Run data folder. "}

	"-"

	"Display Expt ROI Browser", DisplayRatioImgRunBrowserPanel()
	help = {"Displays a control panel used to change the cell or run displayed in a graph."}
		
	"Display Set Calibration Periods", DisplaySetPeriodInfoPanel()
	help = {"Displays a control panel used to set the center time and duration of calibration periods for the run in the active graph."}
	
	"-"
	
	"Load Single-Column Data.../9", LoadSingleROIDataFile()
	help = {"Loads CSV-format data files containing single-wavelength or pre-calculated ratio values."}

	"Display RatioImg Help", DisplayRatioImgHelp()
	help = {"Displays the RatioImg Help file."}
End

// Procedure to read back the cursor position from 2D plots or images
// highly inspirated by "Hook Peak Place" from WaveMetrics
// and JEG ColorLegendTool from Jonathan Guyer
//
// Author: Bela Farago farago@ill.fr

Menu "Graph"
// adds a menu with two hierarchical items to start and stop readback
// this automagicaly ensures we call it only when a Graph window is the target

// (Added keyboard equivalents -MNR)

	Submenu "Cursor Readback"
		"Start Cursor Reading/7",StartCursorReadBack()		
		help = {"Shows x,y(,z) of cursor position"}
		"Stop Cursor Reading/8", StopCursorReadBack()	
		help = {"Stops showing x,y(,z) of cursor position"}
	End
End

function StartCursorReadBack()
// creates a private folder. We use only one string
	if (str2num(StringByKey("IGORVERS" ,IgorInfo(0)))<3.13)
		beep
		abort " Sorry this procedure needs IgorPro version >= 3.13"
	endif
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:ReadBack
	String/G root:Packages:ReadBack:gMessage
	string win=winname(0,1)		// get graph window name	
	if (!IsRatioImgGraph(win))
		DoAlert 0, "Activate a RatioImg graph first."
		return -1
	endif
	if (WinType ("")!= 1)
		DoAlert 0, "Activate a RatioImg graph first."
		return -1
	endif
	setwindow $win,hook = ReadBackHook,hookevents=3,hookcursor=18
	ControlBar 30
	SetVariable msg,pos={10,10},size={545,17},title=" Cursor at:",font="Monaco"
	SetVariable msg,limits={-Inf,Inf,1},value= root:Packages:ReadBack:gMessage
End

function StopCursorReadBack()
// remove readback line
	if (str2num(StringByKey("IGORVERS" ,IgorInfo(0)))<3.13)
		beep
		abort " Sorry this procedure needs IgorPro version >= 3.13"
	endif
string win=winname(0,1)
	setwindow $win,hook = $""
	ControlBar 0
	KillControl msg
End

Function ReadBackHook(s)
	String s
	variable ImReading
	svar msg =  root:Packages:ReadBack:gMessage

	Variable xpix,ypix,xaxval,yaxval,zval
	String ImList

	if( StrSearch(s,"EVENT:mousedown;",0) > 0 )		// this handles mousedown events
		print msg		// on click print in history the latest readback value
		return 1		// don't do normal processing
	endif
	if( StrSearch(s,"EVENT:mousemoved;",0) > 0 )		// readback happens here
		ImList = ImageNameList("",";")
		ImReading = strlen(ImList)				// do we have an image?
		xpix= NumberByKey("MOUSEX",s)
		ypix= NumberByKey("MOUSEY",s)
		xaxval= AxisValFromPixel("","bottom",xpix)
		yaxval= AxisValFromPixel("","left",ypix)
		if (ImReading)							// if we have an image,  z reading is needed
			wave ImageWave =$ StringFromList(0, ImList)	
			zval = ImageWave(xaxval)(yaxval)
			sprintf msg,"x  = %#12.6g, y  = %#12.6g,z  =%#12.6g",xaxval,yaxval,zval
		else
			sprintf msg,"x  = %#12.6g, y  = %#12.6g",xaxval,yaxval
		endif
	endif
	return 0
End

//	InitRatioImgGlobals()
//	Creates and sets global variables used in the rest of the procedures if they do not already exist.
//	If they already exist, it does not change them.
//	The variables and waves created here are described in the Programmer Notes notebook.
Function InitRatioImgGlobals()
	String originalCurrentDataFolder= GetDataFolder(1)

	NewDataFolder/O/S root:Packages
	NewDataFolder/O/S root:Packages:RatioImg
	
	MakeMeasurementWaves(0)   // Create waves for the measurement functions.
 			
	Variable debugMode = NumVarOrDefault("gDebugMode", 0)
	Variable/G gDebugMode = debugMode
	
	// Start of globals used by the LoadRatioImgFile procedure.

	Variable firstDataLine = NumVarOrDefault("gFirstDataLine", 3)		// The number of the first line in the file that contains data. Normally 3.
	Variable/G gFirstDataLine = firstDataLine

	Variable makeTable = NumVarOrDefault("gMakeTable", 0)			// If true, we make a table of the newly loaded data.
	Variable/G gMakeTable = makeTable

	Variable doBkgSub = NumVarOrDefault("gDoBkgSub", 0)				// If true, we do background subtraction for each A/B pair.
	Variable/G gDoBkgSub = doBkgSub

	Variable samplesPerMinute = NumVarOrDefault("gSamplesPerMinute", 6)
	Variable/G gSamplesPerMinute = samplesPerMinute

	// End of globals used by the LoadRatioImgFile procedure.
	
		
	// Start of globals used by the Set Period Info control panel.

	Variable period = NumVarOrDefault("gPeriod", 0)							// The period we are currently setting.
	Variable/G gPeriod = period

	Variable periodCenterTime = NumVarOrDefault("gPeriodCenterTime", 32.5)	// Center time in minutes.
	Variable/G gPeriodCenterTime = periodCenterTime

	Variable periodDuration = NumVarOrDefault("gPeriodDuration", 2)			// Duration time in minutes.
	Variable/G gPeriodDuration = periodDuration

	Variable periodSolution = NumVarOrDefault("gPeriodSolution", 0)			// Calibration solution in millimoles.
	Variable/G gPeriodSolution = periodSolution
	
	Make/O/N=2 centerTimeIndicator = NaN										// Used to show center time of period.

	// End of globals used by the Set Period Info control panel.
	

	// Start of default values used when we create a new run.

	Variable numPeriods = NumVarOrDefault("gNumPeriods", 10)
	Variable/G gNumPeriods = numPeriods

	if (exists("periodInfo") == 0)
		Make/N=(10,6) periodInfo = NaN
		
		periodInfo[0][0] = 32.5			// Center time in minutes.
		periodInfo[0][1] = 2				// Duration in minutes.
		periodInfo[0][2] = 0				// Calibration solution in millimoles.
				
		periodInfo[1][0] = periodInfo[0][0] + 5	
		periodInfo[1][1] = 4						
		periodInfo[1][2] = 15						
			
		periodInfo[2][0] = periodInfo[1][0] + 5
		periodInfo[2][1] = 4
		periodInfo[2][2] = 40	
		
		periodInfo[3][0] = periodInfo[2][0] + 5
		periodInfo[3][1] = 4
		periodInfo[3][2] = 100	
		
		periodInfo[4][0] = periodInfo[3][0] + 5
		periodInfo[4][1] = 4
		periodInfo[4][2] = 40	
		
		periodInfo[5][0] = periodInfo[4][0] + 5
		periodInfo[5][1] = 4
		periodInfo[5][2] = 50	
		
		periodInfo[6][0] = periodInfo[5][0] + 5
		periodInfo[6][1] = 4
		periodInfo[6][2] = 60	

		periodInfo[7][0] = periodInfo[6][0] + 5
		periodInfo[7][1] = 4
		periodInfo[7][2] = 70	
		
		periodInfo[8][0] = periodInfo[7][0] + 5
		periodInfo[8][1] = 4
		periodInfo[8][2] = 80	
		
		periodInfo[9][0] = periodInfo[8][0] + 5
		periodInfo[9][1] = 4
		periodInfo[9][2] = 90	
	endif

	// End of default values used when we create a new run.
	
		SetDataFolder originalCurrentDataFolder
		
	return 0
End

Function RatioImgDebug()
	NVAR debugMode = root:Packages:RatioImg:gDebugMode
	return debugMode
End

//	CreateNewRatioImgDataFolder(dataFolderName)
//	Creates a new data folder with the specified name and makes it the current data folder.
//	Creates the run globals for the new run by copying the defaults from the RatioImg data folder.
Function CreateNewRatioImgDataFolder(dataFolderName)
	String dataFolderName
	
	InitRatioImgGlobals()		// Create defaults if they do not already exist.
	
	NewDataFolder/O/S $dataFolderName					// Create new data folder and make it the current one.

	NVAR defaultSamplesPerMinute = root:Packages:RatioImg:gSamplesPerMinute
	Variable/G gSamplesPerMinute = defaultSamplesPerMinute

	NVAR defaultNumPeriods = root:Packages:RatioImg:gNumPeriods
	Variable/G gNumPeriods = defaultNumPeriods
	
	NVAR doBkgSub = root:Packages:RatioImg:gDoBkgSub
	Variable/G gDoBkgSub = doBkgSub

	Variable/G gRatiosCorrected = 0							// Set to 1 when ratios are corrected.
	
	Duplicate/O root:Packages:RatioImg:periodInfo, periodInfo
	
	return 0
End

Function DisplayRatioImgHelp()
	DisplayHelpTopic "RatioImg: About Ion Ratio Imaging"
End

//	MSetRatioImgDataLoadingSettings(firstDataLine, makeTable, doBkgSub, samplesPerMinute)
//	Displays a dialog in which the user can set various data loading-related settings used in other procedures.
//	This is implemented as a proc (a form of macro) because functions do not have the
//	ability to display the missing parameter dialog.
Proc MSetRatioImgDataLoadingSettings(firstDataLine, makeTable, doBkgSub, samplesPerMinute)
	Variable firstDataLine = root:Packages:RatioImg:gFirstDataLine
	Prompt firstDataLine, "First line in file containing numeric data"
	Variable makeTable = 1 + (root:Packages:RatioImg:gMakeTable == 0)					// In popup menu, the first item is 1.
	Prompt makeTable, "Make a table of data loaded from the file", popup "Yes;No"
	Variable doBkgSub = 1 + (root:Packages:RatioImg:gDoBkgSub == 0)
	Prompt doBkgSub, "Do background subtraction", popup "Yes;No"
	Variable samplesPerMinute = root:Packages:RatioImg:gSamplesPerMinute
	Prompt samplesPerMinute, "Number of samples per minute"
	
	// Store the values that the user entered via the missing parameter dialog in the global variables.	
	root:Packages:RatioImg:gFirstDataLine = firstDataLine
	root:Packages:RatioImg:gMakeTable = makeTable==1
	root:Packages:RatioImg:gDoBkgSub = doBkgSub==1
	root:Packages:RatioImg:gSamplesPerMinute = samplesPerMinute
End


